@@ -8,7 +8,7 @@ import (
 	"fmt"
 	"io"
 	"io/ioutil"
-	"reflect"
+	"net/url"
 	"regexp"
 	"sort"
 	"strings"
@@ -16,6 +16,7 @@ import (
 	"time"
 
 	corev1 "k8s.io/api/core/v1"
+	apixv1beta1client "k8s.io/apiextensions-apiserver/pkg/client/clientset/clientset/typed/apiextensions/v1beta1"
 	policyv1beta1 "k8s.io/api/policy/v1beta1"
 	"k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
@@ -25,7 +26,6 @@ import (
 	"k8s.io/apimachinery/pkg/util/json"
 	utilruntime "k8s.io/apimachinery/pkg/util/runtime"
 	"k8s.io/apimachinery/pkg/util/sets"
-	"k8s.io/client-go/discovery"
 	"k8s.io/client-go/dynamic"
 	kubescheme "k8s.io/client-go/kubernetes/scheme"
 	certificatesv1beta1 "k8s.io/client-go/kubernetes/typed/certificates/v1beta1"
@@ -41,8 +41,6 @@ import (
 	configv1client "github.com/openshift/client-go/config/clientset/versioned/typed/config/v1"
 	imageregistryv1 "github.com/openshift/client-go/imageregistry/clientset/versioned/typed/imageregistry/v1"
 	networkv1client "github.com/openshift/client-go/network/clientset/versioned/typed/network/v1"
-	"github.com/openshift/library-go/pkg/image/reference"
-	_ "k8s.io/apimachinery/pkg/runtime/serializer/yaml"
 
 	"github.com/openshift/insights-operator/pkg/record"
 	"github.com/openshift/insights-operator/pkg/record/diskrecorder"
@@ -58,38 +56,23 @@ const (
 	// will be listed in a single request to reduce memory usage.
 	imageGatherPodLimit = 200
 
-	// containerImageLimit is the maximum number of container images to collect.
-	// On average, information about one image takes up roughly 100 raw bytes.
-	containerImageLimit = 1000
-
-	// yyyyMmDateFormat is the date format used to get a YYYY-MM string.
-	yyyyMmDateFormat = "2006-01"
+	gatherPodDisruptionBudgetLimit = 5000
 
 	// metricsAlertsLinesLimit is the maximal number of lines read from monitoring Prometheus
 	// 500 KiB of alerts is limit, one alert line has typically 450 bytes => 1137 lines.
 	// This number has been rounded to 1000 for simplicity.
 	// Formerly, the `500 * 1024 / 450` expression was used instead.
-	metricsAlertsLinesLimit        = 1000
-	gatherPodDisruptionBudgetLimit = 5000
+	metricsAlertsLinesLimit = 1000
 
 	// csrGatherLimit is the maximum number of crs that
 	// will be listed in a single request to reduce memory usage.
 	csrGatherLimit = 5000
-
-	// Maximal total number of service accounts
-	maxServiceAccountsLimit          = 1000
-	maxServiceAccountNamespacesLimit = 1000
-	maxNamespacesLimit               = 1000
-
-	// InstallPlansTopX is the Maximal number of Install plans by non-unique instances count
-	InstallPlansTopX = 100
 )
 
 var (
-	openshiftSerializer     = openshiftscheme.Codecs.LegacyCodec(configv1.SchemeGroupVersion)
-	kubeSerializer          = kubescheme.Codecs.LegacyCodec(corev1.SchemeGroupVersion)
+	openshiftSerializer = openshiftscheme.Codecs.LegacyCodec(configv1.SchemeGroupVersion)
+	kubeSerializer      = kubescheme.Codecs.LegacyCodec(corev1.SchemeGroupVersion)
 	policyV1Beta1Serializer = kubescheme.Codecs.LegacyCodec(policyv1beta1.SchemeGroupVersion)
-
 	// maxEventTimeInterval represents the "only keep events that are maximum 1h old"
 	// TODO: make this dynamic like the reporting window based on configured interval
 	maxEventTimeInterval = 1 * time.Hour
@@ -102,10 +85,10 @@ var (
 	// logTailLines sets maximum number of lines to fetch from pod logs
 	logTailLines = int64(100)
 
+	imageHostRegex = regexp.MustCompile(`(^|\.)(openshift\.org|registry\.redhat\.io|registry\.access\.redhat\.com)$`)
+
 	// lineSep is the line separator used by the alerts metric
 	lineSep = []byte{'\n'}
-
-	defaultNamespaces = []string{"default", "kube-system", "kube-public"}
 )
 
 func init() {
@@ -117,40 +100,33 @@ func init() {
 
 // Gatherer is a driving instance invoking collection of data
 type Gatherer struct {
-	ctx             context.Context
-	client          configv1client.ConfigV1Interface
-	coreClient      corev1client.CoreV1Interface
-	networkClient   networkv1client.NetworkV1Interface
-	dynamicClient   dynamic.Interface
-	metricsClient   rest.Interface
-	certClient      certificatesv1beta1.CertificatesV1beta1Interface
-	registryClient  imageregistryv1.ImageregistryV1Interface
-	policyClient    policyclient.PolicyV1beta1Interface
-	lock            sync.Mutex
-	lastVersion     *configv1.ClusterVersion
-	discoveryClient discovery.DiscoveryInterface
-}
-
-type clusterOperatorResource struct {
-	APIVersion string      `json:"apiVersion"`
-	Kind       string      `json:"kind"`
-	Name       string      `json:"name"`
-	Spec       interface{} `json:"spec"`
+	ctx            context.Context
+	client         configv1client.ConfigV1Interface
+	coreClient     corev1client.CoreV1Interface
+	networkClient  networkv1client.NetworkV1Interface
+	dynamicClient  dynamic.Interface
+	metricsClient  rest.Interface
+	certClient     certificatesv1beta1.CertificatesV1beta1Interface
+	registryClient imageregistryv1.ImageregistryV1Interface
+	crdClient      apixv1beta1client.ApiextensionsV1beta1Interface
+	policyClient   policyclient.PolicyV1beta1Interface
+	lock           sync.Mutex
+	lastVersion    *configv1.ClusterVersion
 }
 
 // New creates new Gatherer
 func New(client configv1client.ConfigV1Interface, coreClient corev1client.CoreV1Interface, certClient certificatesv1beta1.CertificatesV1beta1Interface, metricsClient rest.Interface,
-	registryClient imageregistryv1.ImageregistryV1Interface, networkClient networkv1client.NetworkV1Interface, dynamicClient dynamic.Interface, policyClient policyclient.PolicyV1beta1Interface, discoveryClient discovery.DiscoveryInterface) *Gatherer {
+	registryClient imageregistryv1.ImageregistryV1Interface, crdClient apixv1beta1client.ApiextensionsV1beta1Interface, networkClient networkv1client.NetworkV1Interface, dynamicClient dynamic.Interface, policyClient policyclient.PolicyV1beta1Interface) *Gatherer {
 	return &Gatherer{
-		client:          client,
-		coreClient:      coreClient,
-		certClient:      certClient,
-		metricsClient:   metricsClient,
-		registryClient:  registryClient,
-		policyClient:    policyClient,
-		networkClient:   networkClient,
-		dynamicClient:   dynamicClient,
-		discoveryClient: discoveryClient,
+		client:         client,
+		coreClient:     coreClient,
+		certClient:     certClient,
+		metricsClient:  metricsClient,
+		registryClient: registryClient,
+		crdClient:      crdClient,
+		networkClient:  networkClient,
+		dynamicClient:  dynamicClient,
+		policyClient:   policyClient,
 	}
 }
 
@@ -178,12 +154,9 @@ func (i *Gatherer) Gather(ctx context.Context, recorder record.Interface) error
 		GatherClusterIngress(i),
 		GatherClusterProxy(i),
 		GatherCertificateSigningRequests(i),
+		GatherCRD(i),
 		GatherHostSubnet(i),
 		GatherMachineSet(i),
-		GatherServiceAccounts(i),
-		GatherMachineConfigPool(i),
-		GatherInstallPlans(i),
-		GatherContainerRuntimeConfig(i),
 	)
 }
 
@@ -196,18 +169,14 @@ func (i *Gatherer) Gather(ctx context.Context, recorder record.Interface) error
 // See: docs/insights-archive-sample/config/pdbs
 func GatherPodDisruptionBudgets(i *Gatherer) func() ([]record.Record, []error) {
 	return func() ([]record.Record, []error) {
-		pdbs, err := i.policyClient.PodDisruptionBudgets("").List(i.ctx, metav1.ListOptions{Limit: gatherPodDisruptionBudgetLimit})
+		pdbs, err := i.policyClient.PodDisruptionBudgets("").List(i.ctx, metav1.ListOptions{Limit: gatherPodDisruptionBudgetLimit,})
 		if err != nil {
 			return nil, []error{err}
 		}
 		records := []record.Record{}
 		for _, pdb := range pdbs.Items {
-			recordName := fmt.Sprintf("config/pdbs/%s", pdb.GetName())
-			if pdb.GetNamespace() != "" {
-				recordName = fmt.Sprintf("config/pdbs/%s/%s", pdb.GetNamespace(), pdb.GetName())
-			}
 			records = append(records, record.Record{
-				Name: recordName,
+				Name: fmt.Sprintf("config/pdbs/%s", pdb.GetName()),
 				Item: PodDisruptionBudgetsAnonymizer{&pdb},
 			})
 		}
@@ -234,6 +203,8 @@ func GatherMostRecentMetrics(i *Gatherer) func() ([]record.Record, []error) {
 		data, err := i.metricsClient.Get().AbsPath("federate").
 			Param("match[]", "etcd_object_counts").
 			Param("match[]", "cluster_installer").
+			Param("match[]", "namespace:container_cpu_usage_seconds_total:sum_rate").
+			Param("match[]", "namespace:container_memory_usage_bytes:sum").
 			DoRaw(i.ctx)
 		if err != nil {
 			// write metrics errors to the file format as a comment
@@ -284,8 +255,12 @@ func GatherContainerImages(i *Gatherer) func() ([]record.Record, []error) {
 	return func() ([]record.Record, []error) {
 		records := []record.Record{}
 
-		// Cache for the temporary image count list.
-		img2month2count := img2Month2CountMap{}
+		contInfo := ContainerInfo{
+			Images:     ContainerImageSet{},
+			Containers: PodsWithAge{},
+		}
+		// Cache for image indices in the collected set of container image.
+		image2idx := map[string]int{}
 
 		// Use the Limit and Continue fields to request the pod information in chunks.
 		continueValue := ""
@@ -304,11 +279,22 @@ func GatherContainerImages(i *Gatherer) func() ([]record.Record, []error) {
 				if strings.HasPrefix(pod.Namespace, "openshift") && hasContainerInCrashloop(podPtr) {
 					records = append(records, record.Record{Name: fmt.Sprintf("config/pod/%s/%s", pod.Namespace, pod.Name), Item: PodAnonymizer{podPtr}})
 				} else if pod.Status.Phase == corev1.PodRunning {
-					startMonth := pod.CreationTimestamp.Time.UTC().Format(yyyyMmDateFormat)
-
-					gatherImages(startMonth, img2month2count, pod.Status.ContainerStatuses)
-					gatherImages(startMonth, img2month2count, pod.Status.InitContainerStatuses)
-					gatherImages(startMonth, img2month2count, pod.Status.EphemeralContainerStatuses)
+					for _, container := range pod.Spec.Containers {
+						imageURL := container.Image
+						urlHost, err := forceParseURLHost(imageURL)
+						if err != nil {
+							klog.Errorf("unable to parse container image URL: %v", err)
+							continue
+						}
+						if imageHostRegex.MatchString(urlHost) {
+							imgIndex, ok := image2idx[imageURL]
+							if !ok {
+								imgIndex = contInfo.Images.Add(imageURL)
+								image2idx[imageURL] = imgIndex
+							}
+							contInfo.Containers.Add(pod.CreationTimestamp.Time, imgIndex)
+						}
+					}
 				}
 			}
 
@@ -320,43 +306,6 @@ func GatherContainerImages(i *Gatherer) func() ([]record.Record, []error) {
 			continueValue = pods.Continue
 		}
 
-		// Transform map into a list for sorting.
-		imageCounts := []tmpImageCountEntry{}
-		for img, countMap := range img2month2count {
-			totalCount := 0
-			for _, count := range countMap {
-				totalCount += count
-			}
-			imageCounts = append(imageCounts, tmpImageCountEntry{
-				Image:         img,
-				TotalCount:    totalCount,
-				CountPerMonth: countMap,
-			})
-		}
-
-		// Sort images from most common to least common.
-		sort.Slice(imageCounts, func(i, j int) bool {
-			return imageCounts[i].TotalCount > imageCounts[j].TotalCount
-		})
-
-		// Reconstruct the image information into the reported data structure.
-		contInfo := ContainerInfo{
-			Images:     ContainerImageSet{},
-			Containers: PodsWithAge{},
-		}
-		totalEntries := 0
-		for _, img := range imageCounts {
-			if totalEntries >= containerImageLimit {
-				break
-			}
-
-			imgIndex := contInfo.Images.Add(img.Image)
-			for month, count := range img.CountPerMonth {
-				contInfo.Containers.Add(month, imgIndex, count)
-				totalEntries++
-			}
-		}
-
 		return append(records, record.Record{
 			Name: "config/running_containers",
 			Item: record.JSONMarshaller{Object: contInfo},
@@ -364,7 +313,7 @@ func GatherContainerImages(i *Gatherer) func() ([]record.Record, []error) {
 	}
 }
 
-// GatherClusterOperators collects all ClusterOperators and their resource.
+// GatherClusterOperators collects all ClusterOperators.
 // It finds unhealthy Pods for unhealthy operators
 //
 // The Kubernetes api https://github.com/openshift/client-go/blob/master/config/clientset/versioned/typed/config/v1/clusteroperator.go#L62
@@ -382,28 +331,9 @@ func GatherClusterOperators(i *Gatherer) func() ([]record.Record, []error) {
 		if err != nil {
 			return nil, []error{err}
 		}
-		resVer, _ := getOperatorResourcesVersions(i.discoveryClient)
 		records := make([]record.Record, 0, len(config.Items))
-		for idx, co := range config.Items {
-			records = append(records, record.Record{Name: fmt.Sprintf("config/clusteroperator/%s", config.Items[idx].Name), Item: ClusterOperatorAnonymizer{&config.Items[idx]}})
-			if resVer == nil {
-				continue
-			}
-			relRes := collectClusterOperatorResources(i.ctx, i.dynamicClient, co, resVer)
-			for _, rr := range relRes {
-				// imageregistry resources (config, pruner) are gathered in image_registries.go, image_pruners.go
-				if strings.Contains(rr.APIVersion, "imageregistry") {
-					continue
-				}
-				gv, err := schema.ParseGroupVersion(rr.APIVersion)
-				if err != nil {
-					klog.Warningf("Unable to parse group version %s: %s", rr.APIVersion, err)
-				}
-				records = append(records, record.Record{
-					Name: fmt.Sprintf("config/clusteroperator/%s/%s/%s", gv.Group, strings.ToLower(rr.Kind), rr.Name),
-					Item: ClusterOperatorResourceAnonymizer{rr},
-				})
-			}
+		for index := range config.Items {
+			records = append(records, record.Record{Name: fmt.Sprintf("config/clusteroperator/%s", config.Items[index].Name), Item: ClusterOperatorAnonymizer{&config.Items[index]}})
 		}
 		namespaceEventsCollected := sets.NewString()
 		now := time.Now()
@@ -466,7 +396,7 @@ func GatherClusterOperators(i *Gatherer) func() ([]record.Record, []error) {
 						logName = fmt.Sprintf("%s_previous.log", c.Name)
 					}
 					buf.Reset()
-					klog.V(2).Infof("Fetching logs for %s container %s pod in namespace %s (previous: %v): %v", c.Name, pod.Name, pod.Namespace, isPrevious, err)
+					klog.V(2).Infof("Fetching logs for %s container %s pod in namespace %s (previous: %v): %q", c.Name, pod.Name, pod.Namespace, isPrevious, err)
 					// Collect container logs and continue on error
 					err = collectContainerLogs(i, pod, buf, c.Name, isPrevious, &bufferSize)
 					if err != nil {
@@ -501,74 +431,6 @@ func collectContainerLogs(i *Gatherer, pod *corev1.Pod, buf *bytes.Buffer, conta
 	return nil
 }
 
-func collectClusterOperatorResources(ctx context.Context, dynamicClient dynamic.Interface, co configv1.ClusterOperator, resVer map[string][]string) []clusterOperatorResource {
-	var relObj []configv1.ObjectReference
-	for _, ro := range co.Status.RelatedObjects {
-		if strings.Contains(ro.Group, "operator.openshift.io") {
-			relObj = append(relObj, ro)
-		}
-	}
-	if len(relObj) == 0 {
-		return nil
-	}
-	var res []clusterOperatorResource
-	for _, ro := range relObj {
-		key := fmt.Sprintf("%s-%s", ro.Group, strings.ToLower(ro.Resource))
-		versions, _ := resVer[key]
-		for _, v := range versions {
-			gvr := schema.GroupVersionResource{Group: ro.Group, Version: v, Resource: strings.ToLower(ro.Resource)}
-			clusterResource, err := dynamicClient.Resource(gvr).Get(ctx, ro.Name, metav1.GetOptions{})
-			if err != nil {
-				klog.V(2).Infof("Unable to list %s resource due to: %s", gvr, err)
-			}
-			if clusterResource == nil {
-				continue
-			}
-			var kind, name, apiVersion string
-			err = failEarly(
-				func() error { return parseJSONQuery(clusterResource.Object, "kind", &kind) },
-				func() error { return parseJSONQuery(clusterResource.Object, "apiVersion", &apiVersion) },
-				func() error { return parseJSONQuery(clusterResource.Object, "metadata.name", &name) },
-			)
-			if err != nil {
-				continue
-			}
-			spec, ok := clusterResource.Object["spec"]
-			if !ok {
-				klog.Warningf("Can't find spec for cluster operator resource %s", name)
-			}
-			res = append(res, clusterOperatorResource{Spec: spec, Kind: kind, Name: name, APIVersion: apiVersion})
-		}
-	}
-	return res
-}
-
-func getOperatorResourcesVersions(discoveryClient discovery.DiscoveryInterface) (map[string][]string, error) {
-	resources, err := discoveryClient.ServerPreferredResources()
-	if err != nil {
-		return nil, err
-	}
-	resourceVersionMap := make(map[string][]string)
-	for _, v := range resources {
-		if strings.Contains(v.GroupVersion, "operator.openshift.io") {
-			gv, err := schema.ParseGroupVersion(v.GroupVersion)
-			if err != nil {
-				continue
-			}
-			for _, ar := range v.APIResources {
-				key := fmt.Sprintf("%s-%s", gv.Group, ar.Name)
-				r, ok := resourceVersionMap[key]
-				if !ok {
-					resourceVersionMap[key] = []string{gv.Version}
-				} else {
-					r = append(r, gv.Version)
-				}
-			}
-		}
-	}
-	return resourceVersionMap, nil
-}
-
 // GatherNodes collects all Nodes.
 //
 // The Kubernetes api https://github.com/kubernetes/client-go/blob/master/kubernetes/typed/core/v1/node.go#L78
@@ -582,10 +444,9 @@ func GatherNodes(i *Gatherer) func() ([]record.Record, []error) {
 			return nil, []error{err}
 		}
 		records := make([]record.Record, 0, len(nodes.Items))
-		for i := range nodes.Items {
-			records = append(records, record.Record{Name: fmt.Sprintf("config/node/%s", nodes.Items[i].Name), Item: NodeAnonymizer{&nodes.Items[i]}})
+		for i, node := range nodes.Items {
+			records = append(records, record.Record{Name: fmt.Sprintf("config/node/%s", node.Name), Item: NodeAnonymizer{&nodes.Items[i]}})
 		}
-
 		return records, nil
 	}
 }
@@ -707,16 +568,17 @@ func GatherClusterNetwork(i *Gatherer) func() ([]record.Record, []error) {
 // Location in archive: config/hostsubnet/
 func GatherHostSubnet(i *Gatherer) func() ([]record.Record, []error) {
 	return func() ([]record.Record, []error) {
-		hsList, err := i.networkClient.HostSubnets().List(i.ctx, metav1.ListOptions{})
+
+		hostSubnetList, err := i.networkClient.HostSubnets().List(i.ctx, metav1.ListOptions{})
 		if errors.IsNotFound(err) {
 			return nil, nil
 		}
 		if err != nil {
 			return nil, []error{err}
 		}
-		records := make([]record.Record, 0, len(hsList.Items))
-		for i, h := range hsList.Items {
-			records = append(records, record.Record{Name: fmt.Sprintf("config/hostsubnet/%s", h.Host), Item: HostSubnetAnonymizer{&hsList.Items[i]}})
+		records := make([]record.Record, 0, len(hostSubnetList.Items))
+		for _, h := range hostSubnetList.Items {
+			records = append(records, record.Record{Name: fmt.Sprintf("config/hostsubnet/%s", h.Host), Item: HostSubnetAnonymizer{&h}})
 		}
 		return records, nil
 	}
@@ -744,7 +606,7 @@ func GatherClusterAuthentication(i *Gatherer) func() ([]record.Record, []error)
 
 // GatherClusterImagePruner fetches the image pruner configuration
 //
-// Location in archive: config/clusteroperator/imageregistry.operator.openshift.io/imagepruner/cluster.json
+// Location in archive: config/imagepruner/
 func GatherClusterImagePruner(i *Gatherer) func() ([]record.Record, []error) {
 	return func() ([]record.Record, []error) {
 		pruner, err := i.registryClient.ImagePruners().Get(i.ctx, "cluster", metav1.GetOptions{})
@@ -754,25 +616,13 @@ func GatherClusterImagePruner(i *Gatherer) func() ([]record.Record, []error) {
 		if err != nil {
 			return nil, []error{err}
 		}
-		// TypeMeta is empty - see https://github.com/kubernetes/kubernetes/issues/3030
-		kinds, _, err := registryScheme.ObjectKinds(pruner)
-		if err != nil {
-			return nil, []error{err}
-		}
-		if len(kinds) > 1 {
-			klog.Warningf("More kinds for image registry pruner operator resource %s", kinds)
-		}
-		objKind := kinds[0]
-		return []record.Record{{
-			Name: fmt.Sprintf("config/clusteroperator/%s/%s/%s", objKind.Group, strings.ToLower(objKind.Kind), pruner.Name),
-			Item: ImagePrunerAnonymizer{pruner},
-		}}, nil
+		return []record.Record{{Name: "config/imagepruner", Item: ImagePrunerAnonymizer{pruner}}}, nil
 	}
 }
 
 // GatherClusterImageRegistry fetches the cluster Image Registry configuration
 //
-// Location in archive: config/clusteroperator/imageregistry.operator.openshift.io/config/cluster.json
+// Location in archive: config/imageregistry/
 func GatherClusterImageRegistry(i *Gatherer) func() ([]record.Record, []error) {
 	return func() ([]record.Record, []error) {
 		config, err := i.registryClient.Configs().Get(i.ctx, "cluster", metav1.GetOptions{})
@@ -782,19 +632,7 @@ func GatherClusterImageRegistry(i *Gatherer) func() ([]record.Record, []error) {
 		if err != nil {
 			return nil, []error{err}
 		}
-		// TypeMeta is empty - see https://github.com/kubernetes/kubernetes/issues/3030
-		kinds, _, err := registryScheme.ObjectKinds(config)
-		if err != nil {
-			return nil, []error{err}
-		}
-		if len(kinds) > 1 {
-			klog.Warningf("More kinds for image registry config operator resource %s", kinds)
-		}
-		objKind := kinds[0]
-		return []record.Record{{
-			Name: fmt.Sprintf("config/clusteroperator/%s/%s/%s", objKind.Group, strings.ToLower(objKind.Kind), config.Name),
-			Item: ImageRegistryAnonymizer{config},
-		}}, nil
+		return []record.Record{{Name: "config/imageregistry", Item: ImageRegistryAnonymizer{config}}}, nil
 	}
 }
 
@@ -887,7 +725,9 @@ func GatherClusterProxy(i *Gatherer) func() ([]record.Record, []error) {
 // Location in archive: config/certificatesigningrequests/
 func GatherCertificateSigningRequests(i *Gatherer) func() ([]record.Record, []error) {
 	return func() ([]record.Record, []error) {
-		requests, err := i.certClient.CertificateSigningRequests().List(i.ctx, metav1.ListOptions{})
+		requests, err := i.certClient.CertificateSigningRequests().List(i.ctx, metav1.ListOptions{
+			Limit: csrGatherLimit,
+		})
 		if errors.IsNotFound(err) {
 			return nil, nil
 		}
@@ -906,6 +746,42 @@ func GatherCertificateSigningRequests(i *Gatherer) func() ([]record.Record, []er
 	}
 }
 
+// GatherCRD collects the specified Custom Resource Definitions.
+//
+// The following CRDs are gathered:
+// - volumesnapshots.snapshot.storage.k8s.io (10745 bytes)
+// - volumesnapshotcontents.snapshot.storage.k8s.io (13149 bytes)
+//
+// The CRD sizes above are in the raw (uncompressed) state.
+//
+// Location in archive: config/crd/
+func GatherCRD(i *Gatherer) func() ([]record.Record, []error) {
+	return func() ([]record.Record, []error) {
+		toBeCollected := []string{
+			"volumesnapshots.snapshot.storage.k8s.io",
+			"volumesnapshotcontents.snapshot.storage.k8s.io",
+		}
+		records := []record.Record{}
+		for _, crdName := range toBeCollected {
+			crd, err := i.crdClient.CustomResourceDefinitions().Get(i.ctx, crdName, metav1.GetOptions{})
+			// Log missing CRDs, but do not return the error.
+			if errors.IsNotFound(err) {
+				klog.V(2).Infof("Cannot find CRD: %q", crdName)
+				continue
+			}
+			// Other errors will be returned.
+			if err != nil {
+				return []record.Record{}, []error{err}
+			}
+			records = append(records, record.Record{
+				Name: fmt.Sprintf("config/crd/%s", crd.Name),
+				Item: record.JSONMarshaller{Object: crd},
+			})
+		}
+		return records, []error{}
+	}
+}
+
 //GatherMachineSet collects MachineSet information
 //
 // The Kubernetes api https://github.com/openshift/machine-api-operator/blob/master/pkg/generated/clientset/versioned/typed/machine/v1beta1/machineset.go
@@ -924,12 +800,8 @@ func GatherMachineSet(i *Gatherer) func() ([]record.Record, []error) {
 		}
 		records := []record.Record{}
 		for _, i := range machineSets.Items {
-			recordName := fmt.Sprintf("machinesets/%s", i.GetName())
-			if i.GetNamespace() != "" {
-				recordName = fmt.Sprintf("machinesets/%s/%s", i.GetNamespace(), i.GetName())
-			}
 			records = append(records, record.Record{
-				Name: recordName,
+				Name: fmt.Sprintf("machinesets/%s", i.GetName()),
 				Item: record.JSONMarshaller{Object: i.Object},
 			})
 		}
@@ -937,168 +809,6 @@ func GatherMachineSet(i *Gatherer) func() ([]record.Record, []error) {
 	}
 }
 
-//GatherMachineConfigPool collects MachineConfigPool information
-//
-// The Kubernetes api https://github.com/openshift/machine-config-operator/blob/master/pkg/apis/machineconfiguration.openshift.io/v1/types.go#L197
-// Response see https://docs.okd.io/latest/rest_api/machine_apis/machineconfigpool-machineconfiguration-openshift-io-v1.html
-//
-// Location in archive: config/machineconfigpools/
-func GatherMachineConfigPool(i *Gatherer) func() ([]record.Record, []error) {
-	return func() ([]record.Record, []error) {
-		mcp := schema.GroupVersionResource{Group: "machineconfiguration.openshift.io", Version: "v1", Resource: "machineconfigpools"}
-		machineCPs, err := i.dynamicClient.Resource(mcp).List(i.ctx, metav1.ListOptions{})
-		if errors.IsNotFound(err) {
-			return nil, nil
-		}
-		if err != nil {
-			return nil, []error{err}
-		}
-		records := []record.Record{}
-		for _, i := range machineCPs.Items {
-			records = append(records, record.Record{
-				Name: fmt.Sprintf("config/machineconfigpools/%s", i.GetName()),
-				Item: record.JSONMarshaller{Object: i.Object},
-			})
-		}
-		return records, nil
-	}
-}
-
-// GatherContainerRuntimeConfig collects ContainerRuntimeConfig  information
-//
-// The Kubernetes api https://github.com/openshift/machine-config-operator/blob/master/pkg/apis/machineconfiguration.openshift.io/v1/types.go#L402
-// Response see https://docs.okd.io/latest/rest_api/machine_apis/containerruntimeconfig-machineconfiguration-openshift-io-v1.html
-//
-// Location in archive: config/containerruntimeconfigs/
-func GatherContainerRuntimeConfig(i *Gatherer) func() ([]record.Record, []error) {
-	return func() ([]record.Record, []error) {
-		crc := schema.GroupVersionResource{Group: "machineconfiguration.openshift.io", Version: "v1", Resource: "containerruntimeconfigs"}
-		containerRCs, err := i.dynamicClient.Resource(crc).List(i.ctx, metav1.ListOptions{})
-		if errors.IsNotFound(err) {
-			return nil, nil
-		}
-		if err != nil {
-			return nil, []error{err}
-		}
-		records := []record.Record{}
-		for _, i := range containerRCs.Items {
-			records = append(records, record.Record{
-				Name: fmt.Sprintf("config/containerruntimeconfigs/%s", i.GetName()),
-				Item: record.JSONMarshaller{Object: i.Object},
-			})
-		}
-		return records, nil
-	}
-}
-
-// GatherInstallPlans collects Top x InstallPlans from all openshift namespaces.
-// Because InstallPlans have unique generated names, it groups them by namespace and the "template"
-// for name generation from field generateName.
-// It also collects Total number of all installplans and all non-unique installplans.
-//
-// The Operators-Framework api https://github.com/operator-framework/api/blob/master/pkg/operators/v1alpha1/installplan_types.go#L26
-//
-// Location in archive: config/installplans/
-func GatherInstallPlans(i *Gatherer) func() ([]record.Record, []error) {
-	return func() ([]record.Record, []error) {
-		var plansBatchLimit int64 = 500
-		cont := ""
-		recs := map[string]*collectedPlan{}
-		total := 0
-		opResource := schema.GroupVersionResource{Group: "operators.coreos.com", Version: "v1alpha1", Resource: "installplans"}
-
-		config, err := getAllNamespaces(i)
-		if errors.IsNotFound(err) {
-			return nil, nil
-		}
-		if err != nil {
-			return nil, []error{err}
-		}
-
-		// collect from all openshift* namespaces
-		for _, ns := range config.Items {
-			if !strings.HasPrefix(ns.Name, "openshift") {
-				continue
-			}
-
-			resInterface := i.dynamicClient.Resource(opResource).Namespace(ns.Name)
-			for {
-				u, err := resInterface.List(i.ctx, metav1.ListOptions{Limit: plansBatchLimit, Continue: cont})
-				if errors.IsNotFound(err) {
-					return nil, nil
-				}
-				if err != nil {
-					return nil, []error{err}
-				}
-				jsonMap := u.UnstructuredContent()
-				var items []interface{}
-				err = failEarly(
-					func() error { return parseJSONQuery(jsonMap, "metadata.continue?", &cont) },
-					func() error { return parseJSONQuery(jsonMap, "items", &items) },
-				)
-				if err != nil {
-					return nil, []error{err}
-				}
-				total += len(items)
-				for _, item := range items {
-					if errs := collectInstallPlan(recs, item); errs != nil {
-						return nil, errs
-					}
-				}
-
-				if cont == "" {
-					break
-				}
-			}
-		}
-
-		return []record.Record{{Name: "config/installplans", Item: InstallPlanAnonymizer{v: recs, total: total}}}, nil
-	}
-}
-
-func collectInstallPlan(recs map[string]*collectedPlan, item interface{}) []error {
-	// Get common prefix
-	csv := "[NONE]"
-	var clusterServiceVersionNames []interface{}
-	var ns, genName string
-	var itemMap map[string]interface{}
-	var ok bool
-	if itemMap, ok = item.(map[string]interface{}); !ok {
-		return []error{fmt.Errorf("cannot cast item to map %v", item)}
-	}
-
-	err := failEarly(
-		func() error {
-			return parseJSONQuery(itemMap, "spec.clusterServiceVersionNames", &clusterServiceVersionNames)
-		},
-		func() error { return parseJSONQuery(itemMap, "metadata.namespace", &ns) },
-		func() error { return parseJSONQuery(itemMap, "metadata.generateName", &genName) },
-	)
-	if err != nil {
-		return []error{err}
-	}
-	if len(clusterServiceVersionNames) > 0 {
-		// ignoring non string
-		csv, _ = clusterServiceVersionNames[0].(string)
-	}
-
-	key := fmt.Sprintf("%s.%s.%s", ns, genName, csv)
-	m, ok := recs[key]
-	if !ok {
-		recs[key] = &collectedPlan{Namespace: ns, Name: genName, CSV: csv, Count: 1}
-	} else {
-		m.Count++
-	}
-	return nil
-}
-
-type collectedPlan struct {
-	Namespace string
-	Name      string
-	CSV       string
-	Count     int
-}
-
 func (i *Gatherer) gatherNamespaceEvents(namespace string) ([]record.Record, []error) {
 	// do not accidentally collect events for non-openshift namespace
 	if !strings.HasPrefix(namespace, "openshift-") {
@@ -1133,126 +843,6 @@ func (i *Gatherer) gatherNamespaceEvents(namespace string) ([]record.Record, []e
 	return []record.Record{{Name: fmt.Sprintf("events/%s", namespace), Item: EventAnonymizer{&compactedEvents}}}, nil
 }
 
-// GatherServiceAccounts collects ServiceAccount stats
-// from kubernetes default and namespaces starting with openshift.
-//
-// The Kubernetes api https://github.com/kubernetes/client-go/blob/master/kubernetes/typed/core/v1/serviceaccount.go#L83
-// Response see https://docs.openshift.com/container-platform/4.3/rest_api/index.html#serviceaccount-v1-core
-//
-// Location of serviceaccounts in archive: config/serviceaccounts
-// See: docs/insights-archive-sample/config/serviceaccounts
-func GatherServiceAccounts(i *Gatherer) func() ([]record.Record, []error) {
-	return func() ([]record.Record, []error) {
-		config, err := i.coreClient.Namespaces().List(i.ctx, metav1.ListOptions{Limit: maxServiceAccountNamespacesLimit})
-		if errors.IsNotFound(err) {
-			return nil, nil
-		}
-		if err != nil {
-			return nil, []error{err}
-		}
-		totalServiceAccounts := 0
-		serviceAccounts := []corev1.ServiceAccount{}
-		records := []record.Record{}
-		namespaces := defaultNamespaces
-		namespaceCollected := sets.NewString()
-		// collect from all openshift* namespaces + kubernetes defaults
-		for _, item := range config.Items {
-			if strings.HasPrefix(item.Name, "openshift") {
-				namespaces = append(namespaces, item.Name)
-			}
-		}
-		for _, namespace := range namespaces {
-			// fetching service accounts from namespace
-			if namespaceCollected.Has(namespace) {
-				continue
-			}
-			svca, err := i.coreClient.ServiceAccounts(namespace).List(i.ctx, metav1.ListOptions{Limit: maxServiceAccountsLimit})
-			if err != nil {
-				klog.V(2).Infof("Unable to read ServiceAccounts in namespace %s error %s", namespace, err)
-				continue
-			}
-
-			totalServiceAccounts += len(svca.Items)
-			for _, j := range svca.Items {
-				if len(serviceAccounts) > maxServiceAccountsLimit {
-					break
-				}
-				serviceAccounts = append(serviceAccounts, j)
-			}
-			namespaceCollected.Insert(namespace)
-		}
-
-		records = append(records, record.Record{Name: fmt.Sprintf("config/serviceaccounts"), Item: ServiceAccountsMarshaller{serviceAccounts, totalServiceAccounts}})
-		return records, nil
-	}
-}
-
-func failEarly(fns ...func() error) error {
-	for _, f := range fns {
-		err := f()
-		if err != nil {
-			return err
-		}
-	}
-	return nil
-}
-
-func parseJSONQuery(j map[string]interface{}, jq string, o interface{}) error {
-	for _, k := range strings.Split(jq, ".") {
-		// optional field
-		opt := false
-		sz := len(k)
-		if sz > 0 && k[sz-1] == '?' {
-			opt = true
-			k = k[:sz-1]
-		}
-
-		if uv, ok := j[k]; ok {
-			if v, ok := uv.(map[string]interface{}); ok {
-				j = v
-				continue
-			}
-			// ValueOf to enter reflect-land
-			dstPtrValue := reflect.ValueOf(o)
-			dstValue := reflect.Indirect(dstPtrValue)
-			dstValue.Set(reflect.ValueOf(uv))
-
-			return nil
-		}
-		if opt {
-			return nil
-		}
-		// otherwise key was not found
-		// keys are case sensitive, because maps are
-		for ki := range j {
-			if strings.ToLower(k) == strings.ToLower(ki) {
-				return fmt.Errorf("key %s wasn't found, but %s was ", k, ki)
-			}
-		}
-		return fmt.Errorf("key %s wasn't found in %v ", k, j)
-	}
-	return fmt.Errorf("query didn't match the structure")
-}
-
-func getAllNamespaces(i *Gatherer) (*corev1.NamespaceList, error) {
-	ns, ok := i.ctx.Value(contextKeyAllNamespaces).(*corev1.NamespaceList)
-	if ok {
-		return ns, nil
-	}
-	ns, err := i.coreClient.Namespaces().List(i.ctx, metav1.ListOptions{Limit: maxNamespacesLimit})
-	if err != nil {
-		return nil, err
-	}
-
-	i.ctx = context.WithValue(i.ctx, contextKeyAllNamespaces, ns)
-
-	return ns, nil
-}
-
-type contextKey string
-
-const contextKeyAllNamespaces contextKey = "allnamespaces"
-
 // RawByte is skipping Marshalling from byte slice
 type RawByte []byte
 
@@ -1709,6 +1299,12 @@ ANONYMIZED
 	return sb.String()
 }
 
+// MinimalNodeInfo contains the most essential information about a node
+type MinimalNodeInfo struct {
+	ProviderID string `json:"providerID"`
+	Image      string `json:"image"`
+}
+
 // RunningImages assigns information about running containers to a specific image index.
 // The index is a reference to an item in the related `ContainerImageSet` instance.
 type RunningImages map[int]int
@@ -1717,15 +1313,17 @@ type RunningImages map[int]int
 type PodsWithAge map[string]RunningImages
 
 // Add inserts the specified container information into the data structure.
-func (p PodsWithAge) Add(startMonth string, image int, count int) {
-	if imageMap, exists := p[startMonth]; exists {
+func (p PodsWithAge) Add(startTime time.Time, image int) {
+	const YyyyMmFormat = "2006-01"
+	month := startTime.UTC().Format(YyyyMmFormat)
+	if imageMap, exists := p[month]; exists {
 		if _, exists := imageMap[image]; exists {
-			imageMap[image] += count
+			imageMap[image]++
 		} else {
-			imageMap[image] = count
+			imageMap[image] = 1
 		}
 	} else {
-		p[startMonth] = RunningImages{image: count}
+		p[month] = RunningImages{image: 1}
 	}
 }
 
@@ -1748,40 +1346,17 @@ type ContainerInfo struct {
 	Containers PodsWithAge       `json:"containers"`
 }
 
-type img2Month2CountMap map[string]map[string]int
-
-type tmpImageCountEntry struct {
-	Image         string
-	CountPerMonth map[string]int
-	TotalCount    int
-}
-
-func gatherImages(startMonth string, img2month2count img2Month2CountMap, containers []corev1.ContainerStatus) {
-	for _, container := range containers {
-		dockerRef, err := reference.Parse(container.Image)
-		if err != nil {
-			klog.Warningf("Unable to parse container image specification: %v", err)
-			continue
-		}
-
-		// Use the sha256 hash ID if available, otherwise use the full image spec.
-		imgMinimal := dockerRef.ID
-		if imgMinimal == "" {
-			imgMinimal = container.Image
-		}
-
-		if countMap, ok := img2month2count[imgMinimal]; ok {
-			var count int
-			if count, ok = countMap[startMonth]; !ok {
-				count = 0
-			}
-			countMap[startMonth] = count + 1
-		} else {
-			img2month2count[imgMinimal] = map[string]int{
-				startMonth: 1,
-			}
-		}
+func forceParseURLHost(rawurl string) (string, error) {
+	// If the scheme isn't specified, the URL will not be parsed nicely and everything will end up in the "path"
+	if !strings.Contains(rawurl, "://") {
+		return forceParseURLHost("https://" + rawurl)
 	}
+
+	parsedURL, err := url.Parse(rawurl)
+	if err != nil {
+		return "", err
+	}
+	return parsedURL.Host, nil
 }
 
 func isContainerInCrashloop(status *corev1.ContainerStatus) bool {
@@ -1858,125 +1433,3 @@ func countLines(r io.Reader) (int, error) {
 		}
 	}
 }
-
-// ServiceAccountsMarshaller implements serialization of Service Accounts
-type ServiceAccountsMarshaller struct {
-	sa                   []corev1.ServiceAccount
-	totalServiceAccounts int
-}
-
-// Marshal implements serialization of ServiceAccount
-func (a ServiceAccountsMarshaller) Marshal(_ context.Context) ([]byte, error) {
-	// Creates map for marshal
-	sr := map[string]interface{}{}
-	st := map[string]interface{}{}
-	st["TOTAL_COUNT"] = a.totalServiceAccounts
-	sr["serviceAccounts"] = st
-	nss := map[string]interface{}{}
-	st["namespaces"] = nss
-	for _, sa := range a.sa {
-		var ns map[string]interface{}
-		var ok bool
-		if _, ok = nss[sa.Namespace]; !ok {
-			ns = map[string]interface{}{}
-			nss[sa.Namespace] = ns
-		} else {
-			ns = nss[sa.Namespace].(map[string]interface{})
-		}
-		ns["name"] = sa.Name
-		ns["secrets"] = len(sa.Secrets)
-	}
-	return json.Marshal(sr)
-}
-
-// GetExtension returns extension for anonymized openshift objects
-func (a ServiceAccountsMarshaller) GetExtension() string {
-	return "json"
-}
-
-// ClusterOperatorResourceAnonymizer implements serialization of clusterOperatorResource
-type ClusterOperatorResourceAnonymizer struct{ resource clusterOperatorResource }
-
-// Marshal serializes clusterOperatorResource with IP address anonymization
-func (a ClusterOperatorResourceAnonymizer) Marshal(_ context.Context) ([]byte, error) {
-	bytes, err := json.Marshal(a.resource)
-	if err != nil {
-		return nil, err
-	}
-	resStr := string(bytes)
-	//anonymize URLs
-	re := regexp.MustCompile(`"(https|http)://(.*?)"`)
-	urlMatches := re.FindAllString(resStr, -1)
-	for _, m := range urlMatches {
-		m = strings.ReplaceAll(m, "\"", "")
-		resStr = strings.ReplaceAll(resStr, m, anonymizeString(m))
-	}
-	// anonymize IP addresses
-	re = regexp.MustCompile(`(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)(\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)){3}`)
-	ipMatches := re.FindAllString(resStr, -1)
-	for _, m := range ipMatches {
-		resStr = strings.ReplaceAll(resStr, m, anonymizeString(m))
-	}
-	return []byte(resStr), nil
-}
-
-// GetExtension returns extension for anonymized cluster operator objects
-func (a ClusterOperatorResourceAnonymizer) GetExtension() string {
-	return "json"
-}
-
-// InstallPlanAnonymizer implements serialization of top x installplans
-type InstallPlanAnonymizer struct {
-	v     map[string]*collectedPlan
-	total int
-	limit int
-}
-
-// Marshal implements serialization of InstallPlan
-func (a InstallPlanAnonymizer) Marshal(_ context.Context) ([]byte, error) {
-	if a.limit == 0 {
-		a.limit = InstallPlansTopX
-	}
-	cnts := []int{}
-	for _, v := range a.v {
-		cnts = append(cnts, v.Count)
-	}
-	sort.Sort(sort.Reverse(sort.IntSlice(cnts)))
-	countLimit := -1
-	if len(cnts) > a.limit && a.limit > 0 {
-		// nth plan is on n-1th position
-		countLimit = cnts[a.limit-1]
-	}
-	// Creates map for marshal
-	sr := map[string]interface{}{}
-	st := map[string]int{}
-	st["TOTAL_COUNT"] = a.total
-	st["TOTAL_NONUNIQ_COUNT"] = len(a.v)
-	sr["stats"] = st
-	uls := 0
-	it := []interface{}{}
-	for _, v := range a.v {
-		if v.Count >= countLimit {
-			kvp := map[string]interface{}{}
-			kvp["ns"] = v.Namespace
-			kvp["name"] = v.Name
-			kvp["csv"] = v.CSV
-			kvp["count"] = v.Count
-			it = append(it, kvp)
-			uls++
-		}
-		if uls >= a.limit {
-			break
-		}
-	}
-	sort.SliceStable(it, func(i, j int) bool {
-		return it[i].(map[string]interface{})["count"].(int) > it[j].(map[string]interface{})["count"].(int)
-	})
-	sr["items"] = it
-	return json.Marshal(sr)
-}
-
-// GetExtension returns extension for anonymized openshift objects
-func (a InstallPlanAnonymizer) GetExtension() string {
-	return "json"
-}
