@@ -8,6 +8,7 @@ import (
 	"time"
 
 	"k8s.io/api/certificates/v1beta1"
+
 	corev1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/util/wait"
@@ -21,6 +22,19 @@ const knownFileSuffixesInsideArchiveRegex string = `(` +
 	`(\/|^)(config|id|invoker|metrics|version)` +
 	`)$`
 
+//https://bugzilla.redhat.com/show_bug.cgi?id=1841057
+func TestUploadNotDelayedAfterStart(t *testing.T) {
+	checkPodsLogs(t, `It is safe to use fast upload`)
+	time1 := logLineTime(t, `Reporting status periodically to .* every`)
+	time2 := logLineTime(t, `Successfully reported id=`)
+	delay := time2.Sub(time1)
+	allowedDelay := time.Minute
+	t.Logf("Archive upload delay was %d seconds", delay/time.Second)
+	if delay > allowedDelay && delay < time.Hour*24-allowedDelay {
+		t.Fatal("Upload after start took too much time")
+	}
+}
+
 // https://bugzilla.redhat.com/show_bug.cgi?id=1750665
 // https://bugzilla.redhat.com/show_bug.cgi?id=1753755
 func TestDefaultUploadFrequency(t *testing.T) {
@@ -74,7 +88,7 @@ func TestDefaultUploadFrequency(t *testing.T) {
 	restartInsightsOperator(t)
 
 	// check logs for "Gathering cluster info every 3m0s"
-	checkPodsLogs(t, "Gathering cluster info every 3m0s")
+	checkPodsLogs(t,"Gathering cluster info every 3m0s")
 }
 
 // TestUnreachableHost checks if insights operator reports "degraded" after 5 unsuccessful upload attempts
@@ -127,7 +141,7 @@ func TestUnreachableHost(t *testing.T) {
 	checkPodsLogs(t, "exceeded than threshold 5. Marking as degraded.")
 
 	// Check the operator is degraded
-	insightsDegraded := operatorConditionCheck(t, clusterOperatorInsights(), "Degraded")
+	insightsDegraded := isOperatorDegraded(t, clusterOperatorInsights())
 	if !insightsDegraded {
 		t.Fatal("Insights is not degraded")
 	}
@@ -138,7 +152,7 @@ func TestUnreachableHost(t *testing.T) {
 	}
 	// Check the operator is not degraded anymore
 	errDegraded := wait.PollImmediate(3*time.Second, 3*time.Minute, func() (bool, error) {
-		insightsDegraded := operatorConditionCheck(t, clusterOperatorInsights(), "Degraded")
+		insightsDegraded := isOperatorDegraded(t, clusterOperatorInsights())
 		if insightsDegraded {
 			return false, nil
 		}
@@ -170,53 +184,21 @@ func latestArchiveContainsConfigMaps(t *testing.T) {
 	}
 }
 
-func latestArchiveContainsNodes(t *testing.T) {
-	Nodes, _ := clientset.CoreV1().Nodes().List(context.Background(), metav1.ListOptions{})
-	if len(Nodes.Items) == 0 {
-		t.Fatal("Nothing to test: api doesn't return any nodes")
-	}
-	for _, node := range Nodes.Items {
-		configMapPath := fmt.Sprintf("^config/node/%s\\.json$", node.Name)
-		err := latestArchiveCheckFiles(t, "node", matchingFileExists, configMapPath)
-		if err != nil {
-			t.Error(err)
-		}
-	}
-}
-
 func TestArchiveContains(t *testing.T) {
 	//https://bugzilla.redhat.com/show_bug.cgi?id=1825756
 	t.Run("ConfigMaps", latestArchiveContainsConfigMaps)
 
-	// not backported to 4.6 yet, uncomment when backported
-	////https://bugzilla.redhat.com/show_bug.cgi?id=1885930
-	//t.Run("ServiceAccounts",
-	//	genLatestArchiveCheckPattern(
-	//		"service accounts", matchingFileExists,
-	//		`^config/serviceaccounts\.json$`))
-
 	//https://bugzilla.redhat.com/show_bug.cgi?id=1834677
 	t.Run("ImageRegistry",
 		genLatestArchiveCheckPattern(
 			"image registry", matchingFileExists,
-			`^config/clusteroperator/imageregistry.operator.openshift.io/config/cluster\.json$`))
+			`^config/imageregistry\.json$`))
 
-	// not backported to 4.5 yet
-	////https://bugzilla.redhat.com/show_bug.cgi?id=1873101
-	//t.Run("SnapshotsCRD",
-	//	genLatestArchiveCheckPattern(
-	//		"snapshots CRD", matchingFileExists,
-	//		`^config/crd/volumesnapshots\.snapshot\.storage\.k8s\.io\.json$`))
-
-	defer ChangeReportTimeInterval(t, 1)()
-	defer degradeOperatorMonitoring(t)()
-
-	checker := LogChecker(t).Timeout(2 * time.Minute)
-	checker.SinceNow().Search(`Recording events/openshift-monitoring`)
-	checker.EnableSinceLastCheck().Search(`Wrote \d+ records to disk in \d+`)
-
-	//https://bugzilla.redhat.com/show_bug.cgi?id=1868165
-	t.Run("Nodes", latestArchiveContainsNodes)
+	//https://bugzilla.redhat.com/show_bug.cgi?id=1879068
+	t.Run("HostsSubnet",
+		genLatestArchiveCheckPattern(
+			"hosts subnet", matchingFileExists,
+			`^config/hostsubnet/ip-.*\.json$`))
 
 	//https://bugzilla.redhat.com/show_bug.cgi?id=1881816
 	t.Run("MachineSet",
@@ -224,22 +206,18 @@ func TestArchiveContains(t *testing.T) {
 			"machine set", matchingFileExists,
 			`^machinesets/.*\.json$`))
 
-	//https://bugzilla.redhat.com/show_bug.cgi?id=1881905
-	t.Run("PodDisruptionBudgets",
+	//https://bugzilla.redhat.com/show_bug.cgi?id=1873101
+	t.Run("SnapshotsCRD",
 		genLatestArchiveCheckPattern(
-			"pod disruption budgets", matchingFileExists,
-			`^config/pdbs/.*\.json$`))
+			"snapshots CRD", matchingFileExists,
+			`^config/crd/volumesnapshots\.snapshot\.storage\.k8s\.io\.json$`))
 
-	t.Run("csr",
-		genLatestArchiveCheckPattern(
-			"csr", matchingFileExists,
-			`^config/certificatesigningrequests/.*\.json$`))
+	defer ChangeReportTimeInterval(t, 1)()
+	defer degradeOperatorMonitoring(t)()
 
-	//https://bugzilla.redhat.com/show_bug.cgi?id=1879068
-	t.Run("HostsSubnet",
-		genLatestArchiveCheckPattern(
-			"hosts subnet", matchingFileExists,
-			`^config/hostsubnet/.*\.json$`))
+	checker := LogChecker(t).Timeout(2*time.Minute)
+	checker.SinceNow().Search(`Recording events/openshift-monitoring`)
+	checker.EnableSinceLastCheck().Search(`Wrote \d+ records to disk in \d+`)
 
 	//https://bugzilla.redhat.com/show_bug.cgi?id=1838973
 	t.Run("Logs",
